@@ -722,6 +722,20 @@ void OS::Initialize(bool hard_abort, const char* const gc_fake_mmap) {
   g_hard_abort = hard_abort;
 }
 
+typedef PVOID (*VirtualAlloc2_t)(HANDLE, PVOID, SIZE_T, ULONG, ULONG,
+                                 MEM_EXTENDED_PARAMETER*, ULONG);
+VirtualAlloc2_t VirtualAlloc2;
+
+void OS::EnsureWin32MemoryAPILoaded() {
+  static bool loaded = false;
+  if (!loaded) {
+    VirtualAlloc2 = (VirtualAlloc2_t)GetProcAddress(
+        GetModuleHandle(L"kernelbase.dll"), "VirtualAlloc2");
+
+    loaded = true;
+  }
+}
+
 // static
 size_t OS::AllocatePageSize() {
   static size_t allocate_alignment = 0;
@@ -801,6 +815,14 @@ DWORD GetProtectionFromMemoryPermission(OS::MemoryPermission access) {
   UNREACHABLE();
 }
 
+void* VirtualAllocWrapper(void* hint, size_t size, DWORD flags, DWORD protect) {
+  if (VirtualAlloc2) {
+    return VirtualAlloc2(nullptr, hint, size, flags, protect, NULL, 0);
+  } else {
+    return VirtualAlloc(hint, size, flags, protect);
+  }
+}
+
 uint8_t* RandomizedVirtualAlloc(size_t size, DWORD flags, DWORD protect,
                                 void* hint) {
   LPVOID base = nullptr;
@@ -816,16 +838,57 @@ uint8_t* RandomizedVirtualAlloc(size_t size, DWORD flags, DWORD protect,
 
   if (use_aslr && protect != PAGE_READWRITE) {
     // For executable or reserved pages try to randomize the allocation address.
-    base = VirtualAlloc(hint, size, flags, protect);
+    base = VirtualAllocWrapper(hint, size, flags, protect);
   }
 
   // On failure, let the OS find an address to use.
   if (base == nullptr) {
-    base = VirtualAlloc(nullptr, size, flags, protect);
+    base = VirtualAllocWrapper(nullptr, size, flags, protect);
   }
   return reinterpret_cast<uint8_t*>(base);
 }
 
+void* AllocateInternal(void* hint, size_t size, size_t alignment,
+                       size_t page_size, DWORD flags, DWORD protect) {
+  // First, try an exact size aligned allocation.
+  uint8_t* base = RandomizedVirtualAlloc(size, flags, protect, hint);
+  if (base == nullptr) return nullptr;  // Can't allocate, we're OOM.
+
+  // If address is suitably aligned, we're done.
+  uint8_t* aligned_base = reinterpret_cast<uint8_t*>(
+      RoundUp(reinterpret_cast<uintptr_t>(base), alignment));
+  if (base == aligned_base) return reinterpret_cast<void*>(base);
+
+  // Otherwise, free it and try a larger allocation.
+  CHECK(VirtualFree(base, 0, MEM_RELEASE));
+
+  // Clear the hint. It's unlikely we can allocate at this address.
+  hint = nullptr;
+
+  // Add the maximum misalignment so we are guaranteed an aligned base address
+  // in the allocated region.
+  size_t padded_size = size + (alignment - page_size);
+  const int kMaxAttempts = 3;
+  aligned_base = nullptr;
+  for (int i = 0; i < kMaxAttempts; ++i) {
+    base = RandomizedVirtualAlloc(padded_size, flags, protect, hint);
+    if (base == nullptr) return nullptr;  // Can't allocate, we're OOM.
+
+    // Try to trim the allocation by freeing the padded allocation and then
+    // calling VirtualAlloc at the aligned base.
+    CHECK(VirtualFree(base, 0, MEM_RELEASE));
+    aligned_base = reinterpret_cast<uint8_t*>(
+        RoundUp(reinterpret_cast<uintptr_t>(base), alignment));
+    base = reinterpret_cast<uint8_t*>(
+        VirtualAllocWrapper(aligned_base, size, flags, protect));
+    // We might not get the reduced allocation due to a race. In that case,
+    // base will be nullptr.
+    if (base != nullptr) break;
+  }
+  DCHECK_IMPLIES(base, base == aligned_base);
+  return reinterpret_cast<void*>(base);
+}
+
 }  // namespace
 
 // static
@@ -842,43 +905,7 @@ void* OS::Allocate(void* hint, size_t size, size_t alignment,
                     : MEM_RESERVE | MEM_COMMIT;
   DWORD protect = GetProtectionFromMemoryPermission(access);
 
-  // First, try an exact size aligned allocation.
-  uint8_t* base = RandomizedVirtualAlloc(size, flags, protect, hint);
-  if (base == nullptr) return nullptr;  // Can't allocate, we're OOM.
-
-  // If address is suitably aligned, we're done.
-  uint8_t* aligned_base = reinterpret_cast<uint8_t*>(
-      RoundUp(reinterpret_cast<uintptr_t>(base), alignment));
-  if (base == aligned_base) return reinterpret_cast<void*>(base);
-
-  // Otherwise, free it and try a larger allocation.
-  CHECK(Free(base, size));
-
-  // Clear the hint. It's unlikely we can allocate at this address.
-  hint = nullptr;
-
-  // Add the maximum misalignment so we are guaranteed an aligned base address
-  // in the allocated region.
-  size_t padded_size = size + (alignment - page_size);
-  const int kMaxAttempts = 3;
-  aligned_base = nullptr;
-  for (int i = 0; i < kMaxAttempts; ++i) {
-    base = RandomizedVirtualAlloc(padded_size, flags, protect, hint);
-    if (base == nullptr) return nullptr;  // Can't allocate, we're OOM.
-
-    // Try to trim the allocation by freeing the padded allocation and then
-    // calling VirtualAlloc at the aligned base.
-    CHECK(Free(base, padded_size));
-    aligned_base = reinterpret_cast<uint8_t*>(
-        RoundUp(reinterpret_cast<uintptr_t>(base), alignment));
-    base = reinterpret_cast<uint8_t*>(
-        VirtualAlloc(aligned_base, size, flags, protect));
-    // We might not get the reduced allocation due to a race. In that case,
-    // base will be nullptr.
-    if (base != nullptr) break;
-  }
-  DCHECK_IMPLIES(base, base == aligned_base);
-  return reinterpret_cast<void*>(base);
+  return AllocateInternal(hint, size, alignment, page_size, flags, protect);
 }
 
 // static
@@ -904,7 +931,7 @@ bool OS::SetPermissions(void* address, size_t size, MemoryPermission access) {
     return VirtualFree(address, size, MEM_DECOMMIT) != 0;
   }
   DWORD protect = GetProtectionFromMemoryPermission(access);
-  return VirtualAlloc(address, size, MEM_COMMIT, protect) != nullptr;
+  return VirtualAllocWrapper(address, size, MEM_COMMIT, protect) != nullptr;
 }
 
 // static
@@ -929,7 +956,7 @@ bool OS::DiscardSystemPages(void* address, size_t size) {
   }
   // DiscardVirtualMemory is buggy in Win10 SP0, so fall back to MEM_RESET on
   // failure.
-  void* ptr = VirtualAlloc(address, size, MEM_RESET, PAGE_READWRITE);
+  void* ptr = VirtualAllocWrapper(address, size, MEM_RESET, PAGE_READWRITE);
   CHECK(ptr);
   return ptr;
 }
@@ -949,6 +976,35 @@ bool OS::DecommitPages(void* address, size_t size) {
   return VirtualFree(address, size, MEM_DECOMMIT) != 0;
 }
 
+// static
+bool OS::CanReserveAddressSpace() { return VirtualAlloc2 != nullptr; }
+
+// static
+Optional<AddressSpaceReservation> OS::CreateAddressSpaceReservation(
+    void* hint, size_t size, size_t alignment,
+    MemoryPermission max_permission) {
+  CHECK(CanReserveAddressSpace());
+
+  size_t page_size = AllocatePageSize();
+  DCHECK_EQ(0, size % page_size);
+  DCHECK_EQ(0, alignment % page_size);
+  DCHECK_LE(page_size, alignment);
+  hint = AlignedAddress(hint, alignment);
+
+  // On Windows, address space reservations are backed by placeholder mappings.
+  void* reservation =
+      AllocateInternal(hint, size, alignment, page_size,
+                       MEM_RESERVE | MEM_RESERVE_PLACEHOLDER, PAGE_NOACCESS);
+  if (!reservation) return {};
+
+  return AddressSpaceReservation(reservation, size);
+}
+
+// static
+bool OS::FreeAddressSpaceReservation(AddressSpaceReservation reservation) {
+  return OS::Free(reservation.base(), reservation.size());
+}
+
 // static
 bool OS::HasLazyCommits() {
   // TODO(alph): implement for the platform.
@@ -1068,6 +1124,64 @@ Win32MemoryMappedFile::~Win32MemoryMappedFile() {
   CloseHandle(file_);
 }
 
+Optional<AddressSpaceReservation> AddressSpaceReservation::CreateSubReservation(
+    void* address, size_t size, OS::MemoryPermission max_permission) {
+  // Nothing to do, the sub reservation must already have been split by now.
+  DCHECK(Contains(address, size));
+  DCHECK_EQ(0, size % OS::AllocatePageSize());
+  DCHECK_EQ(0, reinterpret_cast<uintptr_t>(address) % OS::AllocatePageSize());
+
+  return AddressSpaceReservation(address, size);
+}
+
+bool AddressSpaceReservation::FreeSubReservation(
+    AddressSpaceReservation reservation) {
+  // Nothing to do.
+  // Pages allocated inside the reservation must've already been freed.
+  return true;
+}
+
+bool AddressSpaceReservation::SplitPlaceholder(void* address, size_t size) {
+  DCHECK(Contains(address, size));
+  return VirtualFree(address, size, MEM_RELEASE | MEM_PRESERVE_PLACEHOLDER);
+}
+
+bool AddressSpaceReservation::MergePlaceholders(void* address, size_t size) {
+  DCHECK(Contains(address, size));
+  return VirtualFree(address, size, MEM_RELEASE | MEM_COALESCE_PLACEHOLDERS);
+}
+
+bool AddressSpaceReservation::Allocate(void* address, size_t size,
+                                       OS::MemoryPermission access) {
+  DCHECK(Contains(address, size));
+  CHECK(VirtualAlloc2);
+  DWORD flags = (access == OS::MemoryPermission::kNoAccess)
+                    ? MEM_RESERVE | MEM_REPLACE_PLACEHOLDER
+                    : MEM_RESERVE | MEM_COMMIT | MEM_REPLACE_PLACEHOLDER;
+  DWORD protect = GetProtectionFromMemoryPermission(access);
+  return VirtualAlloc2(nullptr, address, size, flags, protect, NULL, 0);
+}
+
+bool AddressSpaceReservation::Free(void* address, size_t size) {
+  DCHECK(Contains(address, size));
+  return VirtualFree(address, size, MEM_RELEASE | MEM_PRESERVE_PLACEHOLDER);
+}
+
+bool AddressSpaceReservation::SetPermissions(void* address, size_t size,
+                                             OS::MemoryPermission access) {
+  DCHECK(Contains(address, size));
+  return OS::SetPermissions(address, size, access);
+}
+
+bool AddressSpaceReservation::DiscardSystemPages(void* address, size_t size) {
+  DCHECK(Contains(address, size));
+  return OS::DiscardSystemPages(address, size);
+}
+
+bool AddressSpaceReservation::DecommitPages(void* address, size_t size) {
+  DCHECK(Contains(address, size));
+  return OS::DecommitPages(address, size);
+}
 
 // The following code loads functions defined in DbhHelp.h and TlHelp32.h
 // dynamically. This is to avoid being depending on dbghelp.dll and
