@@ -29,6 +29,7 @@
 #include "src/base/base-export.h"
 #include "src/base/build_config.h"
 #include "src/base/compiler-specific.h"
+#include "src/base/optional.h"
 #include "src/base/platform/mutex.h"
 #include "src/base/platform/semaphore.h"
 
@@ -36,6 +37,10 @@
 #include "src/base/qnx-math.h"
 #endif
 
+#if V8_OS_FUCHSIA
+#include <zircon/types.h>
+#endif  // V8_OS_FUCHSIA
+
 #ifdef V8_USE_ADDRESS_SANITIZER
 #include <sanitizer/asan_interface.h>
 #endif  // V8_USE_ADDRESS_SANITIZER
@@ -115,8 +120,11 @@ inline intptr_t InternalGetExistingThreadLocal(intptr_t index) {
 
 #endif  // V8_NO_FAST_TLS
 
+class AddressSpaceReservation;
 class PageAllocator;
 class TimezoneCache;
+class VirtualAddressSpace;
+class VirtualAddressSubspace;
 
 // ----------------------------------------------------------------------------
 // OS
@@ -132,6 +140,17 @@ class V8_BASE_EXPORT OS {
   // - gc_fake_mmap: Name of the file for fake gc mmap used in ll_prof.
   static void Initialize(bool hard_abort, const char* const gc_fake_mmap);
 
+#if V8_OS_WIN
+  // On Windows, ensure the newer memory API is loaded if available.  This
+  // includes function like VirtualAlloc2 and MapViewOfFile3.
+  // TODO(chromium:1218005) this should probably happen as part of Initialize,
+  // but that is currently invoked too late, after the virtual memory cage
+  // is initialized. However, eventually the virtual memory cage initialization
+  // will happen as part of V8::Initialize, at which point this function can
+  // probably be merged into OS::Initialize.
+  static void EnsureWin32MemoryAPILoaded();
+#endif
+
   // Returns the accumulated user time for thread. This routine
   // can be used for profiling. The implementation should
   // strive for high-precision timer resolution, preferable
@@ -291,9 +310,12 @@ class V8_BASE_EXPORT OS {
 
  private:
   // These classes use the private memory management API below.
+  friend class AddressSpaceReservation;
   friend class MemoryMappedFile;
   friend class PosixMemoryMappedFile;
   friend class v8::base::PageAllocator;
+  friend class v8::base::VirtualAddressSpace;
+  friend class v8::base::VirtualAddressSubspace;
 
   static size_t AllocatePageSize();
 
@@ -326,6 +348,15 @@ class V8_BASE_EXPORT OS {
 
   V8_WARN_UNUSED_RESULT static bool DecommitPages(void* address, size_t size);
 
+  V8_WARN_UNUSED_RESULT static bool CanReserveAddressSpace();
+
+  V8_WARN_UNUSED_RESULT static Optional<AddressSpaceReservation>
+  CreateAddressSpaceReservation(void* hint, size_t size, size_t alignment,
+                                MemoryPermission max_permission);
+
+  V8_WARN_UNUSED_RESULT static bool FreeAddressSpaceReservation(
+      AddressSpaceReservation reservation);
+
   static const int msPerSecond = 1000;
 
 #if V8_OS_POSIX
@@ -347,6 +378,73 @@ inline void EnsureConsoleOutput() {
 #endif  // (defined(_WIN32) || defined(_WIN64))
 }
 
+// ----------------------------------------------------------------------------
+// AddressSpaceReservation
+//
+// This class provides the same memory management functions as OS but operates
+// inside a previously reserved contiguous region of virtual address space.
+class V8_BASE_EXPORT AddressSpaceReservation {
+ public:
+  using Address = uintptr_t;
+
+  void* base() const { return base_; }
+  size_t size() const { return size_; }
+
+  bool Contains(void* region_addr, size_t region_size) const {
+    Address base = reinterpret_cast<Address>(base_);
+    Address region_base = reinterpret_cast<Address>(region_addr);
+    return (region_base >= base) &&
+           ((region_base + region_size) <= (base + size_));
+  }
+
+  V8_WARN_UNUSED_RESULT bool Allocate(void* address, size_t size,
+                                      OS::MemoryPermission access);
+
+  V8_WARN_UNUSED_RESULT bool Free(void* address, size_t size);
+
+  V8_WARN_UNUSED_RESULT bool SetPermissions(void* address, size_t size,
+                                            OS::MemoryPermission access);
+
+  V8_WARN_UNUSED_RESULT bool DiscardSystemPages(void* address, size_t size);
+
+  V8_WARN_UNUSED_RESULT bool DecommitPages(void* address, size_t size);
+
+  V8_WARN_UNUSED_RESULT Optional<AddressSpaceReservation> CreateSubReservation(
+      void* address, size_t size, OS::MemoryPermission max_permission);
+
+  V8_WARN_UNUSED_RESULT static bool FreeSubReservation(
+      AddressSpaceReservation reservation);
+
+#if V8_OS_WIN
+  // On Windows, the placeholder mappings backing address space reservations
+  // need to be split and merged as page allocations can only replace an entire
+  // placeholder mapping, not parts of it. This must be done by the users of
+  // this API as it requires a RegionAllocator (or equivalent) to keep track of
+  // sub-regions and decide when to split and when to coalesce multiple free
+  // regions into a single one.
+  V8_WARN_UNUSED_RESULT bool SplitPlaceholder(void* address, size_t size);
+  V8_WARN_UNUSED_RESULT bool MergePlaceholders(void* address, size_t size);
+#endif  // V8_OS_WIN
+
+ private:
+  friend class OS;
+
+#if V8_OS_FUCHSIA
+  AddressSpaceReservation(void* base, size_t size, zx_handle_t vmar)
+      : base_(base), size_(size), vmar_(vmar) {}
+#else
+  AddressSpaceReservation(void* base, size_t size) : base_(base), size_(size) {}
+#endif  // V8_OS_FUCHSIA
+
+  void* base_ = nullptr;
+  size_t size_ = 0;
+
+#if V8_OS_FUCHSIA
+  // On Fuchsia, address space reservations are backed by VMARs.
+  zx_handle_t vmar_ = ZX_HANDLE_INVALID;
+#endif  // V8_OS_FUCHSIA
+};
+
 // ----------------------------------------------------------------------------
 // Thread
 //
