@@ -11,6 +11,7 @@
 namespace v8 {
 
 class ApiFunction;
+class CFunctionInfo;
 
 namespace internal {
 
@@ -24,7 +25,7 @@ class StatsCounter;
 
 #define EXTERNAL_REFERENCE_LIST_WITH_ISOLATE(V)                                \
   V(isolate_address, "isolate")                                                \
-  V(builtins_address, "builtins")                                              \
+  V(builtins_table, "builtins_table")                                          \
   V(handle_scope_implementer_address,                                          \
     "Isolate::handle_scope_implementer_address")                               \
   V(address_of_interpreter_entry_trampoline_instruction_start,                 \
@@ -78,8 +79,16 @@ class StatsCounter;
   V(thread_in_wasm_flag_address_address,                                       \
     "Isolate::thread_in_wasm_flag_address_address")                            \
   V(javascript_execution_assert, "javascript_execution_assert")                \
+  EXTERNAL_REFERENCE_LIST_WITH_ISOLATE_EXTERNAL_CODE_SPACE(V)                  \
   EXTERNAL_REFERENCE_LIST_WITH_ISOLATE_HEAP_SANDBOX(V)
 
+#ifdef V8_EXTERNAL_CODE_SPACE
+#define EXTERNAL_REFERENCE_LIST_WITH_ISOLATE_EXTERNAL_CODE_SPACE(V) \
+  V(builtins_code_data_container_table, "builtins_code_data_container_table")
+#else
+#define EXTERNAL_REFERENCE_LIST_WITH_ISOLATE_EXTERNAL_CODE_SPACE(V)
+#endif  // V8_EXTERNAL_CODE_SPACE
+
 #ifdef V8_HEAP_SANDBOX
 #define EXTERNAL_REFERENCE_LIST_WITH_ISOLATE_HEAP_SANDBOX(V) \
   V(external_pointer_table_address,                          \
@@ -126,6 +135,7 @@ class StatsCounter;
   V(f64_mod_wrapper_function, "f64_mod_wrapper")                               \
   V(get_date_field_function, "JSDate::GetField")                               \
   V(get_or_create_hash_raw, "get_or_create_hash_raw")                          \
+  V(gsab_byte_length, "GsabByteLength")                                        \
   V(ieee754_acos_function, "base::ieee754::acos")                              \
   V(ieee754_acosh_function, "base::ieee754::acosh")                            \
   V(ieee754_asin_function, "base::ieee754::asin")                              \
@@ -155,8 +165,6 @@ class StatsCounter;
   V(jsarray_array_join_concat_to_sequential_string,                            \
     "jsarray_array_join_concat_to_sequential_string")                          \
   V(jsreceiver_create_identity_hash, "jsreceiver_create_identity_hash")        \
-  V(length_tracking_gsab_backed_typed_array_length,                            \
-    "LengthTrackingGsabBackedTypedArrayLength")                                \
   V(libc_memchr_function, "libc_memchr")                                       \
   V(libc_memcpy_function, "libc_memcpy")                                       \
   V(libc_memmove_function, "libc_memmove")                                     \
@@ -303,18 +311,21 @@ class StatsCounter;
 #ifdef V8_INTL_SUPPORT
 #define EXTERNAL_REFERENCE_LIST_INTL(V)                               \
   V(intl_convert_one_byte_to_lower, "intl_convert_one_byte_to_lower") \
-  V(intl_to_latin1_lower_table, "intl_to_latin1_lower_table")
+  V(intl_to_latin1_lower_table, "intl_to_latin1_lower_table")         \
+  V(intl_ascii_collation_weights_l1, "Intl::AsciiCollationWeightsL1") \
+  V(intl_ascii_collation_weights_l3, "Intl::AsciiCollationWeightsL3")
 #else
 #define EXTERNAL_REFERENCE_LIST_INTL(V)
 #endif  // V8_INTL_SUPPORT
 
-#ifdef V8_VIRTUAL_MEMORY_CAGE
+#ifdef V8_CAGED_POINTERS
 #define EXTERNAL_REFERENCE_LIST_VIRTUAL_MEMORY_CAGE(V)               \
   V(virtual_memory_cage_base_address, "V8VirtualMemoryCage::base()") \
-  V(virtual_memory_cage_end_address, "V8VirtualMemoryCage::end()")
+  V(virtual_memory_cage_end_address, "V8VirtualMemoryCage::end()")   \
+  V(empty_backing_store_buffer, "EmptyBackingStoreBuffer()")
 #else
 #define EXTERNAL_REFERENCE_LIST_VIRTUAL_MEMORY_CAGE(V)
-#endif  // V8_VIRTUAL_MEMORY_CAGE
+#endif  // V8_CAGED_POINTERS
 
 #ifdef V8_HEAP_SANDBOX
 #define EXTERNAL_REFERENCE_LIST_HEAP_SANDBOX(V) \
@@ -398,6 +409,15 @@ class ExternalReference {
   static ExternalReference Create(StatsCounter* counter);
   static V8_EXPORT_PRIVATE ExternalReference Create(ApiFunction* ptr,
                                                     Type type);
+  // The following version is used by JSCallReducer in the compiler
+  // to create a reference for a fast API call, with one or more
+  // overloads. In simulator builds, it additionally "registers"
+  // the overloads with the simulator to ensure it maintains a
+  // mapping of callable Address'es to a function signature, encoding
+  // GP and FP arguments.
+  static V8_EXPORT_PRIVATE ExternalReference
+  Create(Isolate* isolate, ApiFunction* ptr, Type type, Address* c_functions,
+         const CFunctionInfo* const* c_signatures, unsigned num_functions);
   static ExternalReference Create(const Runtime::Function* f);
   static ExternalReference Create(IsolateAddressId id, Isolate* isolate);
   static ExternalReference Create(Runtime::FunctionId id);
