@@ -37,7 +37,6 @@ namespace internal {
   T(AwaitNotInAsyncContext,                                                    \
     "await is only valid in async functions and the top level bodies of "      \
     "modules")                                                                 \
-  T(AwaitNotInAsyncFunction, "await is only valid in async function")          \
   T(AwaitNotInDebugEvaluate,                                                   \
     "await can not be used when evaluating code "                              \
     "while paused in the debugger")                                            \
@@ -321,6 +320,9 @@ namespace internal {
     "Invalid property descriptor. Cannot both specify accessors and a value "  \
     "or writable attribute, %")                                                \
   T(VarRedeclaration, "Identifier '%' has already been declared")              \
+  T(VarNotAllowedInEvalScope,                                                  \
+    "Identifier '%' cannot be declared with 'var' in current evaluation "      \
+    "scope, consider trying 'let' instead")                                    \
   T(WrongArgs, "%: Arguments list has wrong type")                             \
   /* ReferenceError */                                                         \
   T(NotDefined, "% is not defined")                                            \
@@ -347,7 +349,7 @@ namespace internal {
   T(InvalidCountValue, "Invalid count value")                                  \
   T(InvalidDataViewAccessorOffset,                                             \
     "Offset is outside the bounds of the DataView")                            \
-  T(InvalidDataViewLength, "Invalid DataView length %")                        \
+  T(InvalidDataViewLength, "Invalid DataView length")                          \
   T(InvalidOffset, "Start offset % is outside the bounds of the buffer")       \
   T(InvalidHint, "Invalid hint: %")                                            \
   T(InvalidIndex, "Invalid value: not (convertible to) a safe integer")        \
@@ -442,7 +444,8 @@ namespace internal {
     "Invalid module export name: contains unpaired surrogate")                 \
   T(InvalidRegExpFlags, "Invalid flags supplied to RegExp constructor '%'")    \
   T(InvalidOrUnexpectedToken, "Invalid or unexpected token")                   \
-  T(InvalidPrivateBrand, "Object must be an instance of class %")              \
+  T(InvalidPrivateBrandInstance, "Receiver must be an instance of class %")    \
+  T(InvalidPrivateBrandStatic, "Receiver must be class %")                     \
   T(InvalidPrivateBrandReinitialization,                                       \
     "Cannot initialize private methods of class % twice on the same object")   \
   T(InvalidPrivateFieldReinitialization,                                       \
@@ -632,7 +635,9 @@ namespace internal {
   T(OptionalChainingNoSuper, "Invalid optional chain from super property")     \
   T(OptionalChainingNoTemplate, "Invalid tagged template on optional chain")   \
   /* AggregateError */                                                         \
-  T(AllPromisesRejected, "All promises were rejected")
+  T(AllPromisesRejected, "All promises were rejected")                         \
+  /* Web snapshots */                                                          \
+  T(WebSnapshotError, "Web snapshot failed: %")
 
 enum class MessageTemplate {
 #define TEMPLATE(NAME, STRING) k##NAME,
