@@ -398,7 +398,7 @@ void Decoder::PrintRvcImm8B(Instruction* instr) {
 void Decoder::PrintRvvVm(Instruction* instr) {
   uint8_t imm = instr->RvvVM();
   if (imm == 0) {
-    out_buffer_pos_ += SNPrintF(out_buffer_ + out_buffer_pos_, "  vm");
+    out_buffer_pos_ += SNPrintF(out_buffer_ + out_buffer_pos_, "  v0.t");
   }
 }
 
@@ -1923,6 +1923,9 @@ void Decoder::DecodeRvvIVV(Instruction* instr) {
     case RO_V_VSSUB_VV:
       Format(instr, "vssub.vv  'vd, 'vs2, 'vs1'vm");
       break;
+    case RO_V_VSSUBU_VV:
+      Format(instr, "vssubu.vv  'vd, 'vs2, 'vs1'vm");
+      break;
     case RO_V_VMIN_VV:
       Format(instr, "vmin.vv   'vd, 'vs2, 'vs1'vm");
       break;
@@ -1992,6 +1995,18 @@ void Decoder::DecodeRvvIVV(Instruction* instr) {
     case RO_V_VNCLIPU_WV:
       Format(instr, "vnclipu.wv 'vd, 'vs2, 'vs1");
       break;
+    case RO_V_VSLL_VV:
+      Format(instr, "vsll.vv   'vd, 'vs2, 'vs1");
+      break;
+    case RO_V_VSRL_VV:
+      Format(instr, "vsrl.vv   'vd, 'vs2, 'vs1");
+      break;
+    case RO_V_VSRA_VV:
+      Format(instr, "vsra.vv   'vd, 'vs2, 'vs1");
+      break;
+    case RO_V_VSMUL_VV:
+      Format(instr, "vsmul.vv  'vd, 'vs2, 'vs1");
+      break;
     default:
       UNSUPPORTED_RISCV();
       break;
@@ -2056,6 +2071,9 @@ void Decoder::DecodeRvvIVI(Instruction* instr) {
     case RO_V_VSRL_VI:
       Format(instr, "vsrl.vi   'vd, 'vs2, 'uimm5'vm");
       break;
+    case RO_V_VSRA_VI:
+      Format(instr, "vsra.vi   'vd, 'vs2, 'uimm5'vm");
+      break;
     case RO_V_VSLL_VI:
       Format(instr, "vsll.vi   'vd, 'vs2, 'uimm5'vm");
       break;
@@ -2184,12 +2202,18 @@ void Decoder::DecodeRvvIVX(Instruction* instr) {
     case RO_V_VSRL_VX:
       Format(instr, "vsrl.vx  'vd, 'vs2, 'rs1");
       break;
+    case RO_V_VSRA_VX:
+      Format(instr, "vsra.vx  'vd, 'vs2, 'rs1");
+      break;
     case RO_V_VNCLIP_WX:
       Format(instr, "vnclip.wx 'vd, 'vs2, 'rs1");
       break;
     case RO_V_VNCLIPU_WX:
       Format(instr, "vnclipu.wx 'vd, 'vs2, 'rs1");
       break;
+    case RO_V_VSMUL_VX:
+      Format(instr, "vsmul.vx  'vd, 'vs2, 'vs1");
+      break;
     default:
       UNSUPPORTED_RISCV();
       break;
@@ -2199,6 +2223,14 @@ void Decoder::DecodeRvvIVX(Instruction* instr) {
 void Decoder::DecodeRvvMVV(Instruction* instr) {
   DCHECK_EQ(instr->InstructionBits() & (kBaseOpcodeMask | kFunct3Mask), OP_MVV);
   switch (instr->InstructionBits() & kVTypeMask) {
+    case RO_V_VMUNARY0: {
+      if (instr->Vs1Value() == VID_V) {
+        Format(instr, "vid.v   'rd, 'vs2'vm");
+      } else {
+        UNSUPPORTED_RISCV();
+      }
+      break;
+    }
     case RO_V_VWXUNARY0:
       if (instr->Vs1Value() == 0x0) {
         Format(instr, "vmv.x.s   'rd, 'vs2");
@@ -2218,6 +2250,50 @@ void Decoder::DecodeRvvMVV(Instruction* instr) {
     case RO_V_VREDMINU:
       Format(instr, "vredminu.vs  'vd, 'vs2, 'vs1'vm");
       break;
+    case RO_V_VXUNARY0:
+      if (instr->Vs1Value() == 0b00010) {
+        Format(instr, "vzext.vf8 'vd, 'vs2'vm");
+      } else if (instr->Vs1Value() == 0b00011) {
+        Format(instr, "vsext.vf8 'vd, 'vs2'vm");
+      } else if (instr->Vs1Value() == 0b00100) {
+        Format(instr, "vzext.vf4 'vd, 'vs2'vm");
+      } else if (instr->Vs1Value() == 0b00101) {
+        Format(instr, "vsext.vf4 'vd, 'vs2'vm");
+      } else if (instr->Vs1Value() == 0b00110) {
+        Format(instr, "vzext.vf2 'vd, 'vs2'vm");
+      } else if (instr->Vs1Value() == 0b00111) {
+        Format(instr, "vsext.vf2 'vd, 'vs2'vm");
+      } else {
+        UNSUPPORTED_RISCV();
+      }
+      break;
+    case RO_V_VWMUL_VV:
+      Format(instr, "vwmul.vv   'vd, 'vs2, 'vs1'vm");
+      break;
+    case RO_V_VWMULU_VV:
+      Format(instr, "vwmulu.vv   'vd, 'vs2, 'vs1'vm");
+      break;
+    case RO_V_VMUL_VV:
+      Format(instr, "vmul.vv   'vd, 'vs2, 'vs1'vm");
+      break;
+    case RO_V_VMULHU_VV:
+      Format(instr, "vmulhu.vv   'vd, 'vs2, 'vs1'vm");
+      break;
+    case RO_V_VDIV_VV:
+      Format(instr, "vdiv.vv   'vd, 'vs2, 'vs1'vm");
+      break;
+    case RO_V_VDIVU_VV:
+      Format(instr, "vdivu.vv   'vd, 'vs2, 'vs1'vm");
+      break;
+    case RO_V_VWADDU_VV:
+      Format(instr, "vwaddu.vv  'vd, 'vs2, 'vs1'vm");
+      break;
+    case RO_V_VWADD_VV:
+      Format(instr, "vwadd.vv  'vd, 'vs2, 'vs1'vm");
+      break;
+    case RO_V_VCOMPRESS_VV:
+      Format(instr, "vcompress.vm 'vd, 'vs2, 'vs1'vm");
+      break;
     default:
       UNSUPPORTED_RISCV();
       break;
@@ -2234,6 +2310,33 @@ void Decoder::DecodeRvvMVX(Instruction* instr) {
         UNSUPPORTED_RISCV();
       }
       break;
+    case RO_V_VWMUL_VX:
+      Format(instr, "vwmul.vx   'vd, 'vs2, 'rs1'vm");
+      break;
+    case RO_V_VWMULU_VX:
+      Format(instr, "vwmulu.vx   'vd, 'vs2, 'rs1'vm");
+      break;
+    case RO_V_VMUL_VX:
+      Format(instr, "vmul.vx   'vd, 'vs2, 'rs1'vm");
+      break;
+    case RO_V_VMULHU_VX:
+      Format(instr, "vmulhu.vx 'vd, 'vs2, 'rs1'vm");
+      break;
+    case RO_V_VDIV_VX:
+      Format(instr, "vdiv.vx   'vd, 'vs2, 'rs1'vm");
+      break;
+    case RO_V_VDIVU_VX:
+      Format(instr, "vdivu.vx  'vd, 'vs2, 'rs1'vm");
+      break;
+    case RO_V_VWADDUW_VX:
+      Format(instr, "vwaddu.wx 'vd, 'vs2, 'rs1'vm");
+      break;
+    case RO_V_VWADDU_VX:
+      Format(instr, "vwaddu.vx 'vd, 'vs2, 'rs1'vm");
+      break;
+    case RO_V_VWADD_VX:
+      Format(instr, "vwadd.vx 'vd, 'vs2, 'rs1'vm");
+      break;
     default:
       UNSUPPORTED_RISCV();
       break;
@@ -2254,12 +2357,33 @@ void Decoder::DecodeRvvFVV(Instruction* instr) {
         case VFNCVT_F_F_W:
           Format(instr, "vfncvt.f.f.w  'vd, 'vs2'vm");
           break;
+        case VFNCVT_X_F_W:
+          Format(instr, "vfncvt.x.f.w  'vd, 'vs2'vm");
+          break;
+        case VFNCVT_XU_F_W:
+          Format(instr, "vfncvt.xu.f.w  'vd, 'vs2'vm");
+          break;
         case VFCVT_F_X_V:
           Format(instr, "vfcvt.f.x.v   'vd, 'vs2'vm");
           break;
         case VFCVT_F_XU_V:
           Format(instr, "vfcvt.f.xu.v  'vd, 'vs2'vm");
           break;
+        case VFWCVT_XU_F_V:
+          Format(instr, "vfwcvt.xu.f.v  'vd, 'vs2'vm");
+          break;
+        case VFWCVT_X_F_V:
+          Format(instr, "vfwcvt.x.f.v   'vd, 'vs2'vm");
+          break;
+        case VFWCVT_F_X_V:
+          Format(instr, "vfwcvt.f.x.v   'vd, 'vs2'vm");
+          break;
+        case VFWCVT_F_XU_V:
+          Format(instr, "vfwcvt.f.xu.v  'vd, 'vs2'vm");
+          break;
+        case VFWCVT_F_F_V:
+          Format(instr, "vfwcvt.f.f.v  'vd, 'vs2'vm");
+          break;
         default:
           UNSUPPORTED_RISCV();
           break;
@@ -2270,6 +2394,9 @@ void Decoder::DecodeRvvFVV(Instruction* instr) {
         case VFCLASS_V:
           Format(instr, "vfclass.v  'vd, 'vs2'vm");
           break;
+        case VFSQRT_V:
+          Format(instr, "vfsqrt.v  'vd, 'vs2'vm");
+          break;
         default:
           break;
       }
@@ -2289,6 +2416,9 @@ void Decoder::DecodeRvvFVV(Instruction* instr) {
     case RO_V_VFMAX_VV:
       Format(instr, "vfmax.vv  'vd, 'vs2, 'vs1'vm");
       break;
+    case RO_V_VFREDMAX_VV:
+      Format(instr, "vfredmax.vs 'vd, 'vs2, 'vs1'vm");
+      break;
     case RO_V_VFMIN_VV:
       Format(instr, "vfmin.vv  'vd, 'vs2, 'vs1'vm");
       break;
