@@ -186,6 +186,9 @@ AllocationResult OldLargeObjectSpace::AllocateRawBackground(
       heap()->incremental_marking()->black_allocation(),
       heap()->incremental_marking()->marking_state()->IsBlack(object));
   page->InitializationMemoryFence();
+  if (identity() == CODE_LO_SPACE) {
+    heap()->isolate()->AddCodeMemoryChunk(page);
+  }
   return object;
 }
 
@@ -264,7 +267,8 @@ void OldLargeObjectSpace::PromoteNewLargeObject(LargePage* page) {
   DCHECK(page->IsLargePage());
   DCHECK(page->IsFlagSet(MemoryChunk::FROM_PAGE));
   DCHECK(!page->IsFlagSet(MemoryChunk::TO_PAGE));
-  size_t object_size = static_cast<size_t>(page->GetObject().Size());
+  PtrComprCageBase cage_base(heap()->isolate());
+  size_t object_size = static_cast<size_t>(page->GetObject().Size(cage_base));
   static_cast<LargeObjectSpace*>(page->owner())->RemovePage(page, object_size);
   page->ClearFlag(MemoryChunk::FROM_PAGE);
   AddPage(page, object_size);
@@ -297,11 +301,12 @@ void LargeObjectSpace::FreeUnmarkedObjects() {
   // Right-trimming does not update the objects_size_ counter. We are lazily
   // updating it after every GC.
   size_t surviving_object_size = 0;
+  PtrComprCageBase cage_base(heap()->isolate());
   while (current) {
     LargePage* next_current = current->next_page();
     HeapObject object = current->GetObject();
     DCHECK(!marking_state->IsGrey(object));
-    size_t size = static_cast<size_t>(object.Size());
+    size_t size = static_cast<size_t>(object.Size(cage_base));
     if (marking_state->IsBlack(object)) {
       Address free_start;
       surviving_object_size += size;
@@ -313,7 +318,7 @@ void LargeObjectSpace::FreeUnmarkedObjects() {
             current->size() - (free_start - current->address());
         heap()->memory_allocator()->PartialFreeMemory(
             current, free_start, bytes_to_free,
-            current->area_start() + object.Size());
+            current->area_start() + object.Size(cage_base));
         size_ -= bytes_to_free;
         AccountUncommitted(bytes_to_free);
       }
@@ -403,7 +408,7 @@ void LargeObjectSpace::Verify(Isolate* isolate) {
     // Byte arrays and strings don't have interior pointers.
     if (object.IsAbstractCode(cage_base)) {
       VerifyPointersVisitor code_visitor(heap());
-      object.IterateBody(map, object.Size(), &code_visitor);
+      object.IterateBody(map, object.Size(cage_base), &code_visitor);
     } else if (object.IsFixedArray(cage_base)) {
       FixedArray array = FixedArray::cast(object);
       for (int j = 0; j < array.length(); j++) {
@@ -517,11 +522,12 @@ void NewLargeObjectSpace::FreeDeadObjects(
   bool is_marking = heap()->incremental_marking()->IsMarking();
   size_t surviving_object_size = 0;
   bool freed_pages = false;
+  PtrComprCageBase cage_base(heap()->isolate());
   for (auto it = begin(); it != end();) {
     LargePage* page = *it;
     it++;
     HeapObject object = page->GetObject();
-    size_t size = static_cast<size_t>(object.Size());
+    size_t size = static_cast<size_t>(object.Size(cage_base));
     if (is_dead(object)) {
       freed_pages = true;
       RemovePage(page, size);
