@@ -49,9 +49,10 @@ MaybeHandle<Object> Runtime::GetObjectProperty(
 
   if (!it.IsFound() && key->IsSymbol() &&
       Symbol::cast(*key).is_private_name()) {
-    MessageTemplate message = Symbol::cast(*key).IsPrivateBrand()
-                                  ? MessageTemplate::kInvalidPrivateBrand
-                                  : MessageTemplate::kInvalidPrivateMemberRead;
+    MessageTemplate message =
+        Symbol::cast(*key).IsPrivateBrand()
+            ? MessageTemplate::kInvalidPrivateBrandInstance
+            : MessageTemplate::kInvalidPrivateMemberRead;
     THROW_NEW_ERROR(isolate, NewTypeError(message, key, lookup_start_object),
                     Object);
   }
@@ -124,11 +125,11 @@ void GeneralizeAllTransitionsToFieldAsMutable(Isolate* isolate, Handle<Map> map,
           DCHECK_EQ(*name, target.GetLastDescriptorName(isolate));
           PropertyDetails details = target.GetLastDescriptorDetails(isolate);
           // Currently, we track constness only for fields.
-          if (details.kind() == kData &&
+          if (details.kind() == PropertyKind::kData &&
               details.constness() == PropertyConstness::kConst) {
             target_maps[target_maps_count++] = handle(target, isolate);
           }
-          DCHECK_IMPLIES(details.kind() == kAccessor,
+          DCHECK_IMPLIES(details.kind() == PropertyKind::kAccessor,
                          details.constness() == PropertyConstness::kConst);
         },
         &no_gc);
@@ -459,7 +460,8 @@ RUNTIME_FUNCTION(Runtime_AddDictionaryProperty) {
   DCHECK(name->IsUniqueName());
 
   PropertyDetails property_details(
-      kData, NONE, PropertyDetails::kConstIfDictConstnessTracking);
+      PropertyKind::kData, NONE,
+      PropertyDetails::kConstIfDictConstnessTracking);
   if (V8_ENABLE_SWISS_NAME_DICTIONARY_BOOL) {
     Handle<SwissNameDictionary> dictionary(
         receiver->property_dictionary_swiss(), isolate);
@@ -777,7 +779,7 @@ RUNTIME_FUNCTION(Runtime_GetProperty) {
         InternalIndex entry = dictionary.FindEntry(isolate, key);
         if (entry.is_found()) {
           PropertyCell cell = dictionary.CellAt(entry);
-          if (cell.property_details().kind() == kData) {
+          if (cell.property_details().kind() == PropertyKind::kData) {
             Object value = cell.value();
             if (!value.IsTheHole(isolate)) return value;
             // If value is the hole (meaning, absent) do the general lookup.
@@ -790,7 +792,7 @@ RUNTIME_FUNCTION(Runtime_GetProperty) {
               lookup_start_object->property_dictionary_swiss();
           InternalIndex entry = dictionary.FindEntry(isolate, *key);
           if (entry.is_found() &&
-              (dictionary.DetailsAt(entry).kind() == kData)) {
+              (dictionary.DetailsAt(entry).kind() == PropertyKind::kData)) {
             return dictionary.ValueAt(entry);
           }
         } else {
@@ -798,7 +800,7 @@ RUNTIME_FUNCTION(Runtime_GetProperty) {
               lookup_start_object->property_dictionary();
           InternalIndex entry = dictionary.FindEntry(isolate, key);
           if ((entry.is_found()) &&
-              (dictionary.DetailsAt(entry).kind() == kData)) {
+              (dictionary.DetailsAt(entry).kind() == PropertyKind::kData)) {
             return dictionary.ValueAt(entry);
           }
         }
@@ -825,11 +827,11 @@ RUNTIME_FUNCTION(Runtime_GetProperty) {
   } else if (lookup_start_obj->IsString() && key_obj->IsSmi()) {
     // Fast case for string indexing using [] with a smi index.
     Handle<String> str = Handle<String>::cast(lookup_start_obj);
-    int index = Handle<Smi>::cast(key_obj)->value();
-    if (index >= 0 && index < str->length()) {
+    int smi_index = Handle<Smi>::cast(key_obj)->value();
+    if (smi_index >= 0 && smi_index < str->length()) {
       Factory* factory = isolate->factory();
       return *factory->LookupSingleCharacterStringFromCode(
-          String::Flatten(isolate, str)->Get(index));
+          String::Flatten(isolate, str)->Get(smi_index));
     }
   }
 
@@ -1093,16 +1095,16 @@ RUNTIME_FUNCTION(Runtime_DefineDataPropertyInLiteral) {
     DCHECK(maybe_vector->IsFeedbackVector());
     Handle<FeedbackVector> vector = Handle<FeedbackVector>::cast(maybe_vector);
     FeedbackNexus nexus(vector, FeedbackVector::ToSlot(index));
-    if (nexus.ic_state() == UNINITIALIZED) {
+    if (nexus.ic_state() == InlineCacheState::UNINITIALIZED) {
       if (name->IsUniqueName()) {
         nexus.ConfigureMonomorphic(name, handle(object->map(), isolate),
                                    MaybeObjectHandle());
       } else {
-        nexus.ConfigureMegamorphic(PROPERTY);
+        nexus.ConfigureMegamorphic(IcCheckType::kProperty);
       }
-    } else if (nexus.ic_state() == MONOMORPHIC) {
+    } else if (nexus.ic_state() == InlineCacheState::MONOMORPHIC) {
       if (nexus.GetFirstMap() != object->map() || nexus.GetName() != *name) {
-        nexus.ConfigureMegamorphic(PROPERTY);
+        nexus.ConfigureMegamorphic(IcCheckType::kProperty);
       }
     }
   }
